// curvature.cpp: implementation of the curvature class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "soft.h"
#include "curvature.h"
#include "Geometry2.h"
#include "math.h"
#include "renderer.h"
#include "Normals.h"
#include "transform.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


static double myPI =  acos(-1.0);

float Tr = 1; 

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
curvature::curvature()
{
  Kmax = NULL;
  Kmin = NULL;
  Tmax = NULL;
  Tmin = NULL;
  SmoothedTmax = NULL;
  SmoothedTmin = NULL;
  ifSmoothedTmax = FALSE;
  ifSmoothedTmin = FALSE;
}
 
curvature::curvature(int numPts)
{
    this->Tmax = new float* [numPts];   
	for (int i=0; i<numPts; i++)
	{
	  this->Tmax[i]= new float[3];
 	}

	this->Tmin = new float* [numPts];   
	for ( i=0; i<numPts; i++ )
	{
	  this->Tmin[i] = new float[3];
	}

    this->Kmax = new float[numPts];
	for ( i=0; i<numPts; i++ )
	{
	  this->Kmax[i] = 0.0;
	}

    this->Kmin = new float[numPts];
	for (i=0; i<numPts; i++)
	{
      this->Kmin[i] = 0.0;
	}

    this->SmoothedTmax = new float* [numPts];   
	for (i=0; i<numPts; i++)
	{
	  this->SmoothedTmax[i]= new float[3];
 	}

	this->SmoothedTmin = new float* [numPts];   
	for ( i=0; i<numPts; i++ )
	{
	  this->SmoothedTmin[i] = new float[3];
	}

	this->size = numPts;

	this->ifSmoothedTmax = FALSE;
    this->ifSmoothedTmin = FALSE;

}

curvature::~curvature()
{
  if (this->Tmax != NULL)
  {
   for (int i=0; i<this->size; i++)
	{
	  delete [] this->Tmax[i];
 	}
	
     delete [] this->Tmax;
  }

  if (this->Tmin != NULL)
  {
   for (int i=0; i<this->size; i++)
   {
     delete [] this->Tmin[i];
   }
    delete [] this->Tmin;
  }

  if (this->Kmax != NULL)
  {
    delete [] this->Kmax;
    this->Kmax = NULL;
  }

  if (this->Kmin != NULL)
  {
    delete [] this->Kmin;
    this->Kmin = NULL;
  }

  if (this->SmoothedTmax != NULL)
  {
   for (int i=0; i<this->size; i++)
	{
	  delete [] this->SmoothedTmax[i];
 	}
	
     delete []this->SmoothedTmax;
  }

  if (this->SmoothedTmin != NULL)
  {
   for (int i=0; i<this->size; i++)
   {
     delete [] this->SmoothedTmin[i];
   }
   delete [] this->SmoothedTmin;
  }

}

void curvature::setModel(polyData *data)
{
  this->model = data;
}

void curvature::getPrincCurv()
{

  int numPts = this->model->numPts;
  int numNei = 0;
  int k, s;

  float *T1 = new float[3];
  float *T2 = new float[3];  

  float scale = 0;
  LList *inc = new LList();
  int *common = new int[10];

  MATRIX3X3 Mvi;

  Normals taubNml = Normals(numPts);
  taubNml.setInput(this->model);
  taubNml.Proceed();

  for (int i=0; i<numPts; i++)                 //for all points
  {
   float Wij;
   float Kij; 
   float Tij[3];
   ZeroMatrix(Mvi);

   inc->Clear();
   int numInc = this->model->incidentVts(i, inc);
   inc->Head();
   for (int incId=0; incId<numInc; incId++)             //for all neighbors
	{ 
	  float Vij[3];
	  int j = inc->Retrieve(); 

	  for (k=0; k<3; k++)
	  {
        Vij[k] = this->model->Points[j][k] - this->model->Points[i][k];
	  }

      scale = Geometry::dot(Vij, taubNml.getOutput()[i]);
	  for (k=0; k<3; k++)
	  {
        Tij[k] =Vij[k] - scale * taubNml.getOutput()[i][k];
	  }

	  Geometry::normalize(Tij);
 	  
	  Kij =0;
	  float normVij = Geometry::Norm(Vij);
	  if (Geometry::isZero(normVij)) normVij=1;
	 
	  Kij = 2.0f*Geometry::dot(taubNml.getOutput()[i],Vij)/(normVij*normVij); 
	
	  //to get weights 	       
	  int edgeNei = this->model->GetEdgeNei(i, j, common);
	
      Wij=0;
 
      for (s=0; s<edgeNei; s++)
	  {
        Wij += this->model->trsArea[common[s]];
  	  }
 
	  Wij /=(2.0f* this->model->nbhdArea[i]);
	  
	  for(int row=0; row<3; row++)
		  for(int col=0; col<3; col++)
			Mvi[row][col] += Wij*Kij*Tij[row]*Tij[col]; 

      inc->Forward();

   }     // for all neighbors
	   
     
      float Wvi[3];
 
	  float E1[3] = {1.0f, 0.0f, 0.0f};

	  float temp1[3], temp2[3];
	  for (k=0; k<3; k++)
	  {
        temp1[k] = E1[k] - taubNml.getOutput()[i][k];
		temp2[k] = E1[k] + taubNml.getOutput()[i][k]; 
	  }

      if (taubNml.getOutput()[i][0]<0)
	  {
	   for (k=0; k<3; k++)
	   {
        Wvi[k] = temp1[k]/(float)sqrt(2.0f*(1-taubNml.getOutput()[i][0]));
	   }
	  }

	  else 
	  {
	   for (k=0; k<3; k++)
	   {
        Wvi[k] = temp2[k]/(float)sqrt(2.0f*(1+taubNml.getOutput()[i][0]));
	   }
	  }

	  MATRIX3X3 Unit;
	  UnitMatrix(Unit);

      MATRIX3X3 Qvi;
	  ZeroMatrix(Qvi);
 
	  for (int row =0; row<3; row++)
		for (int col =0; col<3; col++)
		  {
            Qvi[row][col] = Unit[row][col]-2.0f*Wvi[row]*Wvi[col];
		  }

	  MATRIX3X3 last, tmp, QviT;
	  ZeroMatrix(tmp);
	  ZeroMatrix(last);
	  ZeroMatrix(QviT);

	  TransposeMatrix(QviT, Qvi);

      MultMatrix(tmp, Mvi, Qvi);
	  MultMatrix(last, QviT, tmp);

	  float lambda1, lambda2;

	  float m11 = last[1][1];
	  float m12 = last[2][1];
	  float m22 = last[2][2]; 

      float tempT1[3] = {Qvi[0][1], Qvi[1][1], Qvi[2][1]};
	  float tempT2[3] = {Qvi[0][2], Qvi[1][2], Qvi[2][2]};


	  float myCos, mySin;

	  if (Geometry::isZero(m12))
	  {
		lambda1 = m11;
        lambda2 = m22;
		if (m11=m22) {myCos=mySin=0;}
		else
		{
		  myCos = 1;
		  mySin =0;
		}
	  }
	  else
	  {
	   float Theta = 0.5f*(m22-m11)/m12;

	   float f =  1.0f/(float)(fabs(Theta)+sqrt(Theta*Theta+1));
	   if (Theta <0)  f = -f;

	   myCos = 1.0f/(float)sqrt(f*f+1);
	   mySin = myCos*f;   

	   lambda1 = m11-f*m12;
	   lambda2 = m22+f*m12;
	  } 

	  for (k=0; k<3; k++)
	  {
        T1[k] = myCos*tempT1[k] - mySin*tempT2[k];
		T2[k] = myCos*tempT1[k] + mySin*tempT2[k];
	  }
	
	  float norm1 = Geometry::Norm(T1);
	  float norm2 = Geometry::Norm(T2);
	  if (norm1 ==0) norm1=1;
	  if (norm2 ==0) norm2=1;


	  float Kp1 = 3.0f*lambda1 - lambda2;  // T1
	  float Kp2 = 3.0f*lambda2 - lambda1;  // T2

      if (Kp1 >= Kp2)              
	  {
        this->Kmax[i] = Kp1;
		this->Kmin[i] = Kp2;	
		for (k=0; k<3; k++)
		{
		 this->Tmax[i][k] = T1[k]/norm1;
		 this->Tmin[i][k] = T2[k]/norm2;
		}
	  }
      else if (Kp1<Kp2)
	  {
        this->Kmax[i] = Kp2;
		this->Kmin[i] = Kp1;
		for (k=0; k<3; k++)
		{
		 this->Tmax[i][k] = T2[k]/norm2;
		 this->Tmin[i][k] = T1[k]/norm1;
		}
	  }
 
}                        //for all points
     delete [] T1;
     delete [] T2;
	 delete inc;
	 delete [] common;
}


float curvature::intrpolate(float *A, float *Qi, float *Qii, float K_Qi, float K_Qii)
{
 float 	AQi =0;
 float t;
 float KA;
 float temp[3];

 temp[0] = Qi[0] - A[0];
 temp[1] = Qi[1] - A[1];
 temp[2] = Qi[2] - A[2];

 AQi = Geometry::Norm(temp);

 temp[0] = Qi[0] - Qii[0];
 temp[1] = Qi[1] - Qii[1];
 temp[2] = Qi[2] - Qii[2];
 
 float QiQii = Geometry::Norm(temp);

 t = AQi/QiQii;

 KA = K_Qi*(1-t) + K_Qii*t;

 return KA;
   
}

curvature* curvature::Averaging(int iterN)
{
  curvature tempCurv = curvature(this->size);
  for (int j=0; j<this->size; j++)
  {
    tempCurv.Kmax[j] = this->Kmax[j];
    tempCurv.Kmin[j] = this->Kmin[j];
  }

  curvature smoothCurv = curvature(this->size);
  curvature* smoothP = &smoothCurv;

  LList *inc  = new LList();

  for (int iterCur=0; iterCur < iterN; iterCur ++)
  {
 
   for (int i=0; i<this->size; i++)                 //for all points
   {
    float sumKmax = tempCurv.Kmax[i];
    float sumKmin = tempCurv.Kmin[i];

	inc->Clear();
    int numInc = this->model->incidentVts(i, inc);
	inc->Head();
    for (int incId=0; incId<numInc; incId++)             //for all neighbors
    { 
     sumKmax += tempCurv.Kmax[inc->Retrieve()];
     sumKmin += tempCurv.Kmin[inc->Retrieve()];
	 inc->Forward();
    }

     smoothP->Kmax[i] = sumKmax/numInc;
     smoothP->Kmin[i] = sumKmin/numInc;
  }

   for (int j=0; j<this->size; j++)
   {
    tempCurv.Kmax[j] = smoothP->Kmax[j];
    tempCurv.Kmin[j] = smoothP->Kmin[j];
   }

  }

  inc->Clear();
  delete inc;
  return(smoothP);

}


void curvature::smoothField(float **T, float **smoothed, polyData *model, int iterN)
{ 
   int i;

   if ( iterN == 0 ) 
   {      
	 for (i=0; i<model->numPts; i++ )
	 { 
	  smoothed[i][0] = T[i][0];
	  smoothed[i][1] = T[i][1];
      smoothed[i][2] = T[i][2];
	 }
	  return;
   }

   float *v = new float[3];

   LList *inc = new LList();
   float *ptIdQi = new float[3];

   float **temp;
   temp = new float* [model->numPts];
   for ( i=0; i<model->numPts; i++ )
   {
	 temp[i] = new float[3];
   }

   for (i=0; i<model->numPts; i++ )
   {
	 temp[i][0] = T[i][0];
     temp[i][1] = T[i][1];
     temp[i][2] = T[i][2];
   }

  float minEdge = getMinEdge(model);
  float d;
   int count = 0;
   while (count <iterN )
	{

     for (int ptId=0; ptId<model->numPts; ptId++)
     {
      smoothed[ptId][0] = 0.0;
	  smoothed[ptId][1] = 0.0;
      smoothed[ptId][2] = 0.0;
      
	  inc->Clear();
      int numInc =  model->incidentVts(ptId, inc);
      float total_w =0.0;
	  inc->Head();
	  for (int i=0; i<numInc; i++)
	  {
	    int Qi = inc->Retrieve();

        ptIdQi[0] = model->Points[ptId][0] - model->Points[Qi][0];
        ptIdQi[1] = model->Points[ptId][1] - model->Points[Qi][1];
        ptIdQi[2] = model->Points[ptId][2] - model->Points[Qi][2];

		float nm = Geometry::Norm(ptIdQi);
	    if ( Geometry::isZero(nm) ) nm =1;
		Geometry::cross(temp[ptId],temp[Qi],v);
		float nv =Geometry:: Norm(v);
		if (Geometry::dot(temp[ptId],temp[Qi]) >= 0 )
		{
          d = (float) asin(nv);
		}
        else
		{
	      d = (float) asin( myPI - nv);
		}

		d = d*minEdge/nm;

        float w = (float) exp(- d*d/(Tr*Tr));
	    total_w += w;

        smoothed[ptId][0] += w*temp[Qi][0];
        smoothed[ptId][1] += w*temp[Qi][1];	   
        smoothed[ptId][2] += w*temp[Qi][2];

		inc->Forward();
	  }

	   if (Geometry::isZero(total_w) )
	   {
	    smoothed[ptId][0] = T[ptId][0];
	    smoothed[ptId][1] = T[ptId][1];
        smoothed[ptId][2] = T[ptId][2];
	   }
	   else
	   {
        smoothed[ptId][0] /= total_w;
        smoothed[ptId][1] /= total_w;	   
        smoothed[ptId][2] /= total_w;
	   }
	}                                  // for all points

     for (i=0; i<model->numPts; i++ )
	 {
	  temp[i][0] = smoothed[i][0]/Geometry::Norm(temp[i]);
      temp[i][1] = smoothed[i][1]/Geometry::Norm(temp[i]);
      temp[i][2] = smoothed[i][2]/Geometry::Norm(temp[i]);
     }
 
     count++;
	}                                  // for all iteration

   delete [] v;
   for ( i=0; i<model->numPts; i++ )
   {
	 delete temp[i];
   }
   delete [] temp;
   inc->Clear();
   delete  inc;
   delete [] ptIdQi;
}

