// featureHandler.cpp: implementation of the featureHandler class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "soft.h"
#include "featureHandler.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

const YES = 1;
const NO = 0;

featureHandler::featureHandler()
{
	ridge = LList();
	ravine = LList();
}

featureHandler::~featureHandler()
{
 
}

void featureHandler::getRidge(polyData *model, curvature *C, float threshhold)
{
  float *p1, *p2, *p0;
  float *lambda = new float[1];
  int parallel=0;
  float lineDir[3];
  int ridgePoint;
  int numInc, Id1, Id2;
  float *v = new float[3];

  LList list = LList();
  LList *inc = &list;
 	
  float *A = new float[3];

  for (int i=0; i<model->numPts; i++)                   //for all points
  { 
	if (C->Kmax[i] <0) continue;
    p0 = model->Points[i];
    ridgePoint = YES;
	 
    inc->Clear();
    numInc = model->incidentVts(i, inc);
	inc->Head();
    int isExotic = 0;
    for (int j=0; j<numInc; j++)
	{
	  Id1 = inc->Retrieve();
	  p1 = model->Points[Id1];

	  if (j < numInc -1)
	  { 
  	   inc->Forward();
	   Id2 = inc->Retrieve();
	  }
	  else
	  {
       inc->Head();
       Id2 = inc->Retrieve();
	  }

      p2 = model->Points[Id2];  
	  
      if (  Geometry::ifIntersect(p1, p2, C->Tmin[i], p0) == 1 )
	   {
		 isExotic ++;

		 lineDir[0] = p1[0]-p2[0];
		 lineDir[1] = p1[1]-p2[1];
		 lineDir[2] = p1[2]-p2[2];

         Geometry::line_and_plane(A, lambda, parallel, p1, lineDir, C->Tmin[i], p0);

		 float KA = C->intrpolate(A, p1, p2, C->Kmax[Id1], C->Kmax[Id2]);

		 float Kt = dKmax_dTmin(C, i, model);
		   

		 v[0] = A[0] - p0[0];
		 v[1] = A[1] - p0[1];
		 v[2] = A[2] - p0[2];

		 float PA = (float)Geometry::Norm(v);

		 float newKA =(float) ( KA + 0.5* Kt*Kt *PA*PA /(C->Kmax[i] - C->Kmin[i]));

         if ( (newKA > C->Kmax[i]) ) { ridgePoint = NO; j = numInc;}
	   }      //end ifIntersect
	}  // for all neighbors
  
	
	if ( (ridgePoint == YES) && (C->Kmax[i] > threshhold)) this->ridge.InsertAfter(i); 
  }    // for all point     
  
   delete [] A;
   delete [] v;
   delete [] lambda;
}


void featureHandler::featureConnect(LList *feature, polyData *modelPtr, LList *featureEdges)
{
  LList *inc = new LList();
  int p, p1, p2, numInc, num, count;
  num = feature->length();
  count=0;
  featureEdges->Head();
  while( count<num-1)
  {
    feature->Head();
	for (int temp =0; temp <count; temp++)
	{
     feature->Forward();
	}

	p1 = feature->Retrieve();
	inc->Clear();
    numInc = modelPtr->incidentVts(p1, inc);
	for (int curInc=0; curInc<numInc; curInc++)
	{
      p2 = inc->Retrieve();
      feature->Head();
      for (int j= 0; j<	num; j++)
	  {
       p = feature->Retrieve(); 
	   if ( p2 == p )
	   { 
	     featureEdges->InsertAfter(p1);
         featureEdges->Forward();
         featureEdges->InsertAfter(p2);
         featureEdges->Forward();
	   }
       feature->Forward();
	  }
	 inc->Forward();
	}
    count++;
  }
 
  inc->Clear();
  delete inc;
}  


//
//return a directional direvative up tu sign
//
void featureHandler::getRavine(polyData *model, curvature *C, float threshhold)
{
  float *p1, *p2, *p0;
  float *lambda = new float[1];
  int parallel=0;
  float lineDir[3];
  int ravinePoint;
  int numInc;
  int id1,  id2;

  LList list = LList();
  LList *inc = &list;
  float *A = new float[3];
  float *v = new float[3];

  for (int i=0; i<model->numPts; i++)                   //for all points
  { 
	if (C->Kmin[i] >0) continue;
    p0 = model->Points[i];
    ravinePoint = YES;
    
	inc->Clear();
    numInc = model->incidentVts(i, inc);
    int isExotic = 0;
	inc->Head();
    for (int j=0; j<numInc; j++)
	{
	  id1 = inc->Retrieve();
      if ( j < numInc-1)
	  {
        inc->Forward();
		id2 = inc->Retrieve();
	  }
	  else
	  {
		inc->Head();
		id2 = inc->Retrieve();
	  }

	  p1 = model->Points[id1];
      p2 = model->Points[id2];  
	  
      if (  Geometry::ifIntersect(p1, p2, C->Tmax[i], p0) == 1 )
	   {
		 isExotic ++;

		 lineDir[0] = p1[0]-p2[0];
		 lineDir[1] = p1[1]-p2[1];
		 lineDir[2] = p1[2]-p2[2];

         Geometry::line_and_plane(A, lambda, parallel, p1, lineDir, C->Tmin[i], p0);

		 float KA = C->intrpolate(A, p1, p2, C->Kmin[id1], C->Kmin[id2]);

		 float Kt = dKmin_dTmax(C, i, model);
		   
	
		 v[0] = A[0] - p0[0];
		 v[1] = A[1] - p0[1];
		 v[2] = A[2] - p0[2];

		 float PA = (float)Geometry::Norm(v);

//		 float newKA = (float) (KA + 0.5* Kt*Kt *PA*PA /(C->Kmin[i] - C->Kmax[i]));    
		 float newKA = KA;

         if ( (newKA < C->Kmin[i]) ) { ravinePoint = NO; j = numInc;}
	   }      //end ifIntersect
	}  // for all neighbors
  
	
	if ( (ravinePoint == YES) && (C->Kmin[i] < - threshhold)) 
	{	
	  this->ravine.InsertAfter(i); 	
	}
  }    // for all point     
  
  delete [] A;
  delete [] v;
  delete [] lambda;
}

float featureHandler:: dKmax_dTmin(curvature *Curv, int ptId, polyData *modelPtr)
{ 
  float *p1, *p2, *p0;
  float *lambda = new float[1];
  int parallel =0;
  float lineDir[3];
  int numInc;
  float der;
  int id1, id2;
  float *temp = new float[3];

  LList list = LList();
  LList *inc = &list;
 
  float CD[2][2];
  float K_CD[2];

  p0 = modelPtr->Points[ptId];
  inc->Clear();
  numInc = modelPtr->incidentVts(ptId, inc);
  int i=0;
  inc->Head();
  for (int j=0; j<numInc; j++)
  {
	id1 = inc->Retrieve();
    if ( j < numInc-1)
	 {
       inc->Forward();
	   id2 = inc->Retrieve();
	 }
	 else
	  {
		inc->Head();
		id2 = inc->Retrieve();
	  }

	  p1 = modelPtr->Points[id1];
      p2 = modelPtr->Points[id2];  
	  

    if (  Geometry::ifIntersect(p1, p2, Curv->Tmax[ptId], p0) == 1 )
	{
		lineDir[0] = p1[0]-p2[0];
		lineDir[1] = p1[1]-p2[1];
		lineDir[2] = p1[2]-p2[2];

        Geometry::line_and_plane(CD[i], lambda, parallel, p1, lineDir, Curv->Tmax[ptId], p0);

        K_CD[i] = Curv->intrpolate(CD[i], p1, p2, Curv->Kmax[id1], Curv->Kmax[id2]);
	  	i++;
		if (i==2) j = numInc;
	}


	temp[0] = p0[0] - CD[0][0];
	temp[1] = p0[1] - CD[0][1];
	temp[2] = p0[2] - CD[0][2];

	float p0C =(float)Geometry::Norm(temp);

	temp[0] = p0[0] - CD[1][0];
	temp[1] = p0[1] - CD[1][1];
	temp[2] = p0[2] - CD[1][2];

	float p0D =(float)Geometry::Norm(temp);

    der = (K_CD[0] - Curv->Kmax[ptId])/p0C +
            (Curv->Kmax[ptId] - K_CD[1])/p0D -
					  (K_CD[0] - K_CD[1])/(p0C+p0D);
  }
  

  delete [] temp;
  delete [] lambda;
 
  return der;
}

float featureHandler:: dKmin_dTmax(curvature *Curv, int ptId, polyData *modelPtr)
{ 
  float *p1, *p2, *p0;
  float *lambda = new float[1];
  int parallel =0;
  float lineDir[3];
  int numInc;
  float der;
  int id1, id2;
  float *temp = new float[3];

  LList list = LList();
  LList *inc = &list;
 
  float CD[2][2];
  float K_CD[2];

  p0 = modelPtr->Points[ptId];
  inc->Clear();
  numInc = modelPtr->incidentVts(ptId, inc);
  int i=0;
  inc->Head();
  for (int j=0; j<numInc; j++)
  {
	id1 = inc->Retrieve();
    if ( j < numInc-1)
	 {
       inc->Forward();
	   id2 = inc->Retrieve();
	 }
	 else
	  {
		inc->Head();
		id2 = inc->Retrieve();
	  }

	  p1 = modelPtr->Points[id1];
      p2 = modelPtr->Points[id2];  
	  

    if ( Geometry::ifIntersect(p1, p2, Curv->Tmin[ptId], p0) == 1 )
	{
		lineDir[0] = p1[0]-p2[0];
		lineDir[1] = p1[1]-p2[1];
		lineDir[2] = p1[2]-p2[2];

        Geometry::line_and_plane(CD[i], lambda, parallel, p1, lineDir, Curv->Tmin[ptId], p0);

        K_CD[i] = Curv->intrpolate(CD[i], p1, p2, Curv->Kmin[id1], Curv->Kmin[id2]);
	  	i++;
		if (i==2) j = numInc;
	}


	temp[0] = p0[0] - CD[0][0];
	temp[1] = p0[1] - CD[0][1];
	temp[2] = p0[2] - CD[0][2];

	float p0C =(float)Geometry::Norm(temp);

	temp[0] = p0[0] - CD[1][0];
	temp[1] = p0[1] - CD[1][1];
	temp[2] = p0[2] - CD[1][2];

	float p0D =(float)Geometry::Norm(temp);

    der = (K_CD[0] - Curv->Kmin[ptId])/p0C +
            (Curv->Kmin[ptId] - K_CD[1])/p0D -
					  (K_CD[0] - K_CD[1])/(p0C+p0D);
  }
  

  delete [] temp;
  delete [] lambda;
 
  return der;
}


void featureHandler::getNeibhdTrs(LList feature, polyData *model, LList *neibhdTrs )
{

  if (neibhdTrs->length() != 0 )
	  neibhdTrs->Clear();
  neibhdTrs->Head();

  int temp, i;
  LList *inc = new LList();

  int nPts =feature.length(); 

  //find a number of edges in the feature 1-neighbouhod. 
  //The edge number is equal to the new points  number
  int numEdges =0 ;
  feature.Head();
  for (i=0; i<nPts; i++)
  {
	int ptId = feature.Retrieve();
    temp = model->incidentVts(ptId, inc);
	numEdges += temp;
	inc->Clear();
	feature.Forward();
  }

  delete inc;

  numEdges /= 2;    

// find number of triangles in neibhd
//
 int *nei = new int[5];
  
 for (i=0; i<nPts; i++)
  {
	int ptId = feature.Retrieve();
 
	for (int cur=0; cur<model->Neis[ptId].numNei; cur++)
	{
      int trId = model->Neis[ptId].pointNei[cur];
      
	  int p0 = model->GetPointId(trId, 0);
	  int p1 = model->GetPointId(trId, 1);
	  int p2 = model->GetPointId(trId, 2);

	  if (p0<p1)
	  {
        int t = model->GetEdgeNei(p0, p1, nei);
		neibhdTrs->InsertAfter(nei[0]);
		neibhdTrs->Forward();
		neibhdTrs->InsertAfter(nei[1]);
	  }

	  if (p1<p2)
	  {
        int t = model->GetEdgeNei(p0, p1, nei);
		neibhdTrs->InsertAfter(nei[0]);
		neibhdTrs->Forward();
		neibhdTrs->InsertAfter(nei[1]);
	  }

	  if (p2<p0)
	  {
        int t = model->GetEdgeNei(p0, p1, nei);
		neibhdTrs->InsertAfter(nei[0]);
		neibhdTrs->Forward();
		neibhdTrs->InsertAfter(nei[1]);
	  }
	}          //for 1-circle

    feature.Forward();
  }

  delete [] nei;

}

