// fileManager.cpp: implementation of the fileManager class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "soft.h"
#include "fileManager.h"
#include "polyData.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

fileManager::fileManager()
{
 
}

fileManager::~fileManager()
{

}

void fileManager::setFile(CString fileName, CString fileType)
{
    fileName.MakeLower();
	this->fileName = fileName;
    this->fileType = fileType;
}


BOOL fileManager::readFileToModel(CString fileName, polyData *model)
{

  if(this->fileType == "ply2")
	{
	  this->readPly2(fileName, model);
	}

  if (this->fileType == "vtk")
	{
	  if (!this->readVTK(fileName, model))
		  return FALSE;
	}

  return TRUE;
}

BOOL  fileManager::readVTK(CString fileName, polyData *model)
{
	int nPts, nPolys;

	ifstream inFile;
    inFile.open(fileName); 
    if (!inFile) 
	{ 
	 AfxMessageBox("Can not open File:" + fileName, MB_OK); 
     return FALSE;
	}

  char line[256];
  int num_line = 0;
  int count = 0;
  char *p=NULL; 
  int curPoint=0;

  char pattern1[] = "POINTS";
  char pattern2[] ="POLYGONS";

inFile.getline(line, sizeof(line));
strlwr(line);

CString sample = "# vtk DataFile Version"; 
sample.MakeLower();

if (strnicmp(line,  sample, 22) != 0)
	{ 
	 AfxMessageBox("Unrecognisable file format:" + fileName, MB_OK); 
     return FALSE;
	}


inFile.getline(line, sizeof(line));
while ( (p=strstr(line, pattern1)) == 0)
{ 
  inFile.getline(line,sizeof(line) );
} 

char text[10]; 
int temp;
  
if ( (p=strstr(line, pattern1)) != NULL)
 {
  sscanf(line, "%s %d %d", &text, &nPts, &temp);
 }  // read the number of points


  model->SetNumPoints(nPts);
  model->allocatePoints(nPts);

//to know number of points at a line

 inFile.getline(line, sizeof(line));
 sample  = line;
 sample.TrimRight();
 sample.TrimLeft();

 int mode = 9;

 int c =0;
 while (1)
 {
   int place = sample.Find(" ");
   if (place == -1)
   {
	 mode = 3; break;
   }
   else
    sample.SetAt(place, '1');
    c++; 
    if (c>3) break;
 }


 switch (mode)
 {
  case 3:
	  { 
       sscanf(line, "%f %f %f ", &model->Points[curPoint][0],
	                        &model->Points[curPoint][1], 
		                    &model->Points[curPoint][2]);
       curPoint++;

       while (inFile.getline(line, sizeof(line)))
	   { 
         if ((p=strstr(line, pattern2)) != NULL) break;

        	sscanf(line, "%f %f %f ", &model->Points[curPoint][0],
	                        &model->Points[curPoint][1], 
		                    &model->Points[curPoint][2]);
         curPoint++;
	   }
	   break;
	  }

  case 9:

   sscanf(line, "%f %f %f %f %f %f %f %f %f", &model->Points[curPoint][0],
	                                          &model->Points[curPoint][1], 
		                                      &model->Points[curPoint][2],
										      &model->Points[curPoint+1][0],
										      &model->Points[curPoint+1][1],
										      &model->Points[curPoint+1][2],
										      &model->Points[curPoint+2][0],
											  &model->Points[curPoint+2][1],
											  &model->Points[curPoint+2][2]); 
   curPoint +=3; 
  

  while (inFile.getline(line, sizeof(line)))
  { 
  if ((p=strstr(line, pattern2)) != NULL) break;


   sscanf(line, "%f %f %f %f %f %f %f %f %f", &model->Points[curPoint][0],
	                                          &model->Points[curPoint][1], 
		                                      &model->Points[curPoint][2],
										      &model->Points[curPoint+1][0],
										      &model->Points[curPoint+1][1],
										      &model->Points[curPoint+1][2],
										      &model->Points[curPoint+2][0],
											  &model->Points[curPoint+2][1],
											  &model->Points[curPoint+2][2]); 
   curPoint +=3; 
  
};  //end of search for "POLYGONS" and reading points data

 } //edn of swith

if ( (p=strstr(line, pattern2)) != NULL)
 {
  sscanf(line, "%s %d %d", &text, &nPolys, &temp);
 }  // read the number of polygons
else nPolys=0;

   model->SetNumTrs(nPolys);


if (nPolys !=0)
{

  model->allocatePolys(nPolys);
  int temp;

  int curTr=0;

  while (inFile.getline(line, sizeof(line)))
  { 
    sscanf(line, "%d %d %d %d", &temp, &model->Triangles[curTr][0],
	      &model->Triangles[curTr][1], &model->Triangles [curTr][2]);
    curTr++; 
    if (curTr == nPolys) break;
  }  //end of triangles reading

}

inFile.clear();
inFile.close(); //clear stream state and close it

return TRUE;
}

void fileManager::readPly2(CString fileName, polyData *model)
{
	int nPts, nPolys;

	FILE* file = fopen(fileName,"r");

	fscanf(file, "%d", &nPts);
	fscanf(file, "%d", &nPolys);

    model->SetNumPoints(nPts);
    model->allocatePoints(nPts);

    model->SetNumTrs(nPolys);
    model->allocatePolys(nPolys);

	float p[3];
	float dummy;
	for(int i=0; i<nPts; i++){
		fscanf(file, "%f", &p[0]);
		fscanf(file, "%f", &p[2]);
		fscanf(file, "%f", &p[1]);
	    model->setPoint(i, p);

	}

	int f[3];
	for(i=0; i<nPolys; i++){
		fscanf(file, "%f", &dummy);
		fscanf(file, "%d", &f[0]);
		fscanf(file, "%d", &f[1]);
		fscanf(file, "%d", &f[2]);
		
		model->Triangles[i][0] =f[0];
		model->Triangles[i][1] =f[1];
		model->Triangles[i][2] =f[2];
	}

	fclose(file);
}

