// polyData.cpp: implementation of the polyData class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "soft.h"
#include "polyData.h"
#include "Geometry2.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

polyData::polyData()
{
  this->Points = NULL;
  this->Triangles = NULL;
  this->trsArea = NULL;
  this->nbhdArea = NULL;
  this->Neis = NULL;  
  this->numTrs = 0;
  this->numPts = 0;
}

polyData::polyData(int numPts, int numTrs)
{
//initialization of point data

   	this->Points = new float* [numPts];   //
	for ( int i=0; i<numPts; i++ )
	{
	  this->Points[i] = new float[3];
	}

//initialization of triangle data

	this->Triangles = new int* [numTrs];   //
	for ( i=0; i<numTrs; i++ )
	{
	  this->Triangles[i] = new int[3];
	}

   this->Neis = new IncidentTrs[numPts];    
   for ( i=0; i<numPts; i++)
   {
     this->Neis[i].pointNei = new int[100];  
     this->Neis[i].numNei = 0;
   }
 
//number of point and triangles
	this->numTrs = numTrs;
	this->numPts = numPts;

	this->box = boundbox(); 

// sum of the surface areas of all the triangles incident to a vertex

    this->nbhdArea = new float[numPts];
	for ( i=0; i<numPts; i++ )
	{
	  this->nbhdArea[i] = 0.0;
	}

    this->trsArea = new float[numTrs];
	for (i=0; i<numTrs; i++)
	{
      this->trsArea[i] = 0.0;
	}

}

polyData::~polyData()
{   
  if (this->Points != NULL)
  {
   for ( int i=0; i<numPts; i++ )
	{
	  delete [] this->Points[i];
	}
    delete [] this->Points;
  }

  if (this->Triangles)
  {
   for ( int i=0; i<numTrs; i++ )
	{
	  delete [] this->Triangles[i];
	}

   delete [] this->Triangles;
  }
   
  if (this->Neis)
  {
   
  for (int i=0; i<numPts; i++)
   {
     delete [] this->Neis[i].pointNei;  
   }

    delete [] this->Neis;
  }
  
  if (this->nbhdArea)
  {
   delete [] nbhdArea; 
   this->nbhdArea = NULL;
  }

  if (this->trsArea)
  {
   delete [] trsArea;
   this->trsArea = NULL;
  }

}
 
void polyData::SetNumPoints(int numPts)
{
  this->numPts = numPts;
}

void polyData::SetNumTrs(int numTrs)
{
  this->numTrs = numTrs;
}

void polyData::allocatePoints(int numPts)
{
//initialization of point data

    if (this->Points != NULL)
	{	
	 for (int i=0; i<numPts; i++)
	 {
       delete this->Points[i];
	 }
	 delete [] this->Points;
	}

    this->Points = new float* [numPts];   //
	for (int i=0; i<numPts; i++)
	{
	  this->Points[i]= new float[3];
 	}

}
  
void polyData::allocatePolys(int numTrs)
{
//initialization of triangle data

    if (this->Triangles  != NULL)
		delete [] this->Triangles;

	this->Triangles = new int* [numTrs];   //
	for ( int i=0; i<numTrs; i++ )
	{
	  this->Triangles[i] = new int[3];
	}
}
 

void polyData::setPoint(int i, float p[3])
{
  this->Points[i][0] = p[0];
  this->Points[i][1] = p[1];
  this->Points[i][2] = p[2];
}

void polyData::setTriangle(int i, int tr[3])
{
  this->Triangles[i][0] = tr[0];
  this->Triangles[i][1] = tr[1];
  this->Triangles[i][2] = tr[2];
}

void polyData::makeBoundBox()
{ 

  this->box.Xmax = this->box.Xmin =0.0;
  this->box.Ymax = this->box.Ymin =0.0;
  this->box.Zmax = this->box.Zmin =0.0;  

  float x_max, x_min;
  float y_max, y_min;
  float z_max, z_min;

  x_max = x_min = this->Points[0][0];
  y_max = y_min = this->Points[0][1];
  z_max = z_min = this->Points[0][2];

  for (int i=0; i<this->numPts; i++)
  {
    if (x_min> this->Points[i][0] ) x_min = this->Points[i][0];
    if (x_max< this->Points[i][0] ) x_max = this->Points[i][0];

    if (y_min> this->Points[i][1] ) y_min = this->Points[i][1];
    if (y_max< this->Points[i][1] ) y_max = this->Points[i][1];

    if (z_min> this->Points[i][2] ) z_min = this->Points[i][2];
    if (z_max< this->Points[i][2] ) z_max = this->Points[i][2];  
  }

  this->box.Xmin = x_min;
  this->box.Xmax = x_max;

  this->box.Ymin = y_min;
  this->box.Ymax = y_max;

  this->box.Zmin = z_min;
  this->box.Zmax = z_max;
  
}

//
// output: at erach vertex we got all incident
// triangles and number of triangles (starting from 1) 
//
void polyData::BuildLinks( )
{

   if (this->Neis != NULL)
       delete [] this->Neis;

   this->Neis = new IncidentTrs[numPts];    
   for (int i=0; i<numPts; i++)
   {
     this->Neis[i].pointNei = new int[100];  
     this->Neis[i].numNei = 0;
   }

   for (int trId=0; trId<numTrs; trId++)
	{
      for (int k=0; k<3; k++)
	  { 
		int thisVert = this->GetPointId(trId, k);
		this->Neis[thisVert].pointNei[this->Neis[thisVert].numNei] = trId;
		this->Neis[thisVert].numNei++;        
       }
	}

}

int polyData::GetPointId(int trId, int trVertId)
{
  return this->Triangles[trId][trVertId];
}

void polyData::GetPoint(int trId, int trVertId, float *p)
{ 
  p[0] = this->Points[this->GetPointId(trId, trVertId)][0];
  p[1] = this->Points[this->GetPointId(trId, trVertId)][1];
  p[2] = this->Points[this->GetPointId(trId, trVertId)][2]; 
}


// p1 and p2 are the point Id numbers
// return  !! All !! triangles Id 
//
int polyData::GetEdgeNei(int p1, int p2, int *nei)
{

  int numEdgeNei=0;
  int j=0;
   
  for (int neiId=0; neiId <this->Neis[p1].numNei ; neiId++)
  {
    int v1, v2, v3;
    v1 = this->GetPointId(this->Neis[p1].pointNei[neiId], 0);
    v2 = this->GetPointId(this->Neis[p1].pointNei[neiId], 1);
    v3 = this->GetPointId(this->Neis[p1].pointNei[neiId], 2);

	if ( ((v1 == p1) || (v2 == p1 )|| (v3 == p1)) && 
		 ((v1 == p2) || (v2 == p2) || (v3 == p2)) )
	{
	  nei[j] = this->Neis[p1].pointNei[neiId];
	  j++;
      numEdgeNei++;
	}

  }
  return numEdgeNei;
}

BOOL polyData::isBoundaryEdge(int p1, int p2)
{
	int *nei = new int[5];

    int c = GetEdgeNei(p1, p2, nei);

	delete [] nei;

	if (c ==1 ) return TRUE;
	else return FALSE;

}


void polyData::getAreas()
{   
	int i;
    
	if(this->nbhdArea == NULL)
	{
      this->nbhdArea = new float[numPts];
	}

	if (!this->trsArea)
	{
      this->trsArea = new float[numTrs];
	}

	 for (i=0; i<numTrs; i++)
	 {
      this->trsArea[i] = 0.0;
	 }

	 for (i=0; i<numPts; i++ )
	 {
	  this->nbhdArea[i] = 0.0;
	 }
	

   int trId, k;
   int pointId0, pointId1, pointId2;
   float p_0[3], p_1[3], p_2[3]
	  ;
   for (trId=0; trId<this->numTrs; trId++)
   {
	pointId0 = this->GetPointId(trId,0);
	pointId1 = this->GetPointId(trId,1);
	pointId2 = this->GetPointId(trId,2);

	for(k=0;k<3;k++)
	{
  	 p_0[k]= this->Points[pointId0][k];
	 p_1[k] = this->Points[pointId1][k];
	 p_2[k] = this->Points[pointId2][k];  
	}

	float a1[3], a2[3], c[3];

        //   a1/\a2
		 //   /____\
         // p_1  b  p_2
		
    for(k=0;k<3;k++)
	{
	  a1[k] = p_1[k]- p_0[k];
      a2[k] = p_2[k]- p_0[k];
	}

    Geometry::cross(a1,a2,c);                  // c = a1 x a2
   	
	this->trsArea[trId] = (float)(0.5*Geometry::Norm(c)) ;
  }

  

  for (int ptId=0; ptId<this->numPts; ptId++)
  {
    int numNei = this->Neis[ptId].numNei;
    for (int neiId=0; neiId<numNei; neiId++)
    {
	 int trId = this->Neis[ptId].pointNei[neiId];
     this->nbhdArea[ptId] += this->trsArea[trId];
	}
  }

}

int polyData::ifConsistient(int trId1, int vert1, 
							     int vert2, int trId2)
{ 
  int p1_inTr1 = 0;
  int p2_inTr1 = 0;

  int p1_inTr2 = 0;
  int p2_inTr2 = 0;

  int p1, p2;
  int k;

  p1 = this->GetPointId(trId1, vert1);
  p2 = this->GetPointId(trId1, vert2);

  k =0;  
  while ( this->GetPointId(trId1, k)!= p1 ) k++;
  if ( this->GetPointId(trId1, (k+1)%3) == p2)  p1_inTr1 = 1;

  k = 0;
  while ( this->GetPointId(trId2, k)!= p1 ) k++;
  if ( this->GetPointId(trId2, (k+1)%3) == p2)  p1_inTr2 = 1;

  if ( p1_inTr1 == p1_inTr2 ) return 0;
  else return 1;

}

void polyData::reverseTr(int trId)
{
  int temp0 = this->GetPointId(trId, 0);

  this->Triangles[trId][0] = this->Triangles[trId][2];
  this->Triangles[trId][2] = temp0;

}

 static int NOT_VISITED = 0;
 static int NEED_VISITING = 1;
 static int VISITED =2;
 static int *Visited;  //the visited array keeps  track of 
                            //which polygons have been visited

void polyData::orient()
{   
  int numTrs = this->numTrs;
  int i;
  int trId;
  int noCellsNeedVisiting;

  static int size = 1;
  static int temp_size = 0;
  static int numVisited = 0;

  Visited = new int[numTrs];
  for ( i=0; i<numTrs; i++)
  {
    Visited[i] = NOT_VISITED;
  }

  for (trId=0; trId<numTrs; trId++)
  {
    noCellsNeedVisiting = 1;

	if ( Visited[trId] == NOT_VISITED )
	{
	  if ( this->TraversAndOrder(trId) )
	  {
        noCellsNeedVisiting = 0;
	  }
	}

	while (!noCellsNeedVisiting )
	{
      noCellsNeedVisiting = 1;
      for (int i=0; i<numTrs; i++)
	  {
        if ( Visited[i] == NEED_VISITING )
		{
			if ( this->TraversAndOrder(i) )
			{
              noCellsNeedVisiting = 0;
			}
		}
	  }
	}
  }
  delete [] Visited;
}

int polyData::TraversAndOrder(int trId)
{
   int p1, p2;
   int numEdgeNei;
   int *edgeNeighbors = new int[5];
   int queuedCells =0;

   Visited[trId] = VISITED;
   for (int j=0; j<3; j++)
   {
     p1 = this->GetPointId(trId, j);
	 p2 = this->GetPointId(trId, (j+1)%3);

	 numEdgeNei = GetEdgeNei(p1, p2, edgeNeighbors);
     //
	 // check the direction of  neighbor ordering
	 //
     for (int k=0; k< numEdgeNei; k++)
	 {
	   int neighbor = edgeNeighbors[k];
       if ( neighbor == trId) continue;
	   
	   if ( Visited[neighbor] == NOT_VISITED )
	   {
		 for (int l=0; l<3; l++)
		 {
		   if ( this->GetPointId(neighbor, l) == p2 )
		   {
			   break;
		   }
		 }

		 if ( this->GetPointId(neighbor, (l+1)%3) != p1)
		 {
		   this->reverseTr(neighbor);
		 }

		 Visited[neighbor] = NEED_VISITING;
         queuedCells = 1;
	   }    
	 }    // for each edge neighbor
   }    // for all edges of this polygon

   delete edgeNeighbors;
   return queuedCells;
}

// result: ordered 1st neighbors
//  
int polyData::incidentVts(int ptId, LList *nei)
 {
   int incSize = 0;
   int curPt = 0;
   int trId = 0;

   int *inc= new int[100];

   int numNei = this->Neis[ptId].numNei;
   for (int neiId=0; neiId<numNei; neiId++)
   {
	 trId = this->Neis[ptId].pointNei[neiId];
	 for (int k=0; k<3; k++)
	 {
	  int vertex = this->GetPointId(trId, k);
	  if ( vertex != ptId )
	  {
	   int WAS = 0;
       curPt = 0;
       while(curPt <incSize)
	   {
		  if ( inc[curPt] == vertex ) 
		  {
			WAS = 1; break;
		  }
		  else curPt++;
	   }
	   if ( !WAS ) 
	   {
	    inc[curPt] = vertex;
		incSize++;
	   }
	  }   
	 }

   }
 
  
   // now want to order them
   //

  for (int i=0; i<incSize; i++)
  {
   int temp, j; 
   int p2;
   int p1 = inc[i];
   for (j = i+1; j<incSize; j++)
   {
     p2 = inc[j];
     if ( isEdge(p1, p2) )  break;
   }
   if (j != i+1)
   {   
	temp = inc[i+1];
    inc[i+1] = p2;
    inc[j] = temp;
   }
  }

  if (nei->length != 0)
	  nei->Clear();

  //to put them on the list
  nei->Head();  
  for ( i=0; i<incSize; i++)
  {
    nei->InsertAfter(inc[i]);
	nei->Forward();
  }

  delete [] inc;

  return incSize;  

 }

int polyData::isEdge(int p1, int p2)
{
  int result = 0;
  int numNei = this->Neis[p1].numNei;
  for (int neiId=0; neiId<numNei; neiId++)
   {
	 int trId = this->Neis[p1].pointNei[neiId];
	 for (int k=0; k<3; k++)
	 {
	  int vertex = this->GetPointId(trId, k);
	  if ( vertex == p2 ) { result =1; break; break;}
	 }
  }
   return result;
}

void polyData::ReverseOrientation()
{
  for (int i=0; i<this->numTrs; i++)
  {
   this->reverseTr(i);
  }
}


void  polyData::Translate2Origin()
{
  float newX, newY, newZ;
  float Xc, Yc, Zc;       //centre of bounding box

  this->makeBoundBox();

  Xc = (float) 0.5*(this->box.Xmax + this->box.Xmin);
  Yc = (float) 0.5*(this->box.Ymax + this->box.Ymin);
  Zc = (float) 0.5*(this->box.Zmax + this->box.Zmin);

  for (int i=0; i<this->numPts; i++)
  {
    newX = this->Points[i][0] - Xc;

    newY = this->Points[i][1] - Yc;

    newZ = this->Points[i][2] - Zc;

    this->Points[i][0] = newX;
    this->Points[i][1] = newY;
    this->Points[i][2] = newZ;
  }

  this->makeBoundBox();
  
}

// normals point inside
void  polyData::InitModel()
{

  float v0[3], v1[3], v2[3];
  int ptId0, ptId1, ptId2;

  if (this == NULL) return;
  
  this->Translate2Origin();
  this->makeBoundBox();
  this->BuildLinks();
  this->orient();

// to make normals to point inside 
  float Vol = 0;
  for (int trId=0; trId<this->numTrs; trId++)
  {
    ptId0 = this->GetPointId(trId, 0);
    ptId1 = this->GetPointId(trId, 1);
    ptId2 = this->GetPointId(trId, 2);

	for (int k=0; k<3; k++)
	{
      v0[k] = this->Points[ptId0][k];
      v1[k] = this->Points[ptId1][k];
      v2[k] = this->Points[ptId2][k];
	}
    
    Vol += ( v0[0]*v1[1]*v2[2] + v0[1]*v1[2]*v2[0] +
             v0[2]*v1[0]*v2[1] - v0[2]*v1[1]*v2[0] -
             v0[0]*v1[2]*v2[1] - v0[1]*v1[0]*v2[2]);	
  }

  if (Vol > 0) this->ReverseOrientation();

  this->getAreas();
  
}

void  polyData::copyPointsFrom(polyData *dataIn)
{

  if  (this->Points == NULL)
  {
    this->SetNumPoints(dataIn->numPts);
    this->allocatePoints(dataIn->numPts); 
  }

  else if
	  (this->numPts != dataIn->numPts) return;

  for (int i=0; i<dataIn->numPts; i++)
	{
	  this->setPoint(i, dataIn->Points[i]);
 	}
}


void  polyData::copyTrianglesFrom(polyData *dataIn)
{
  this->SetNumTrs(dataIn->numTrs);
  this->allocatePolys(dataIn->numTrs);
  for (int i=0; i<dataIn->numTrs; i++ )
	{
	  this->setTriangle(i, dataIn->Triangles[i]);
	}
}



void polyData::trlCircleNei(int trId, LList* nei)
{
	 if (nei->length != 0) nei->Clear();

	 nei->Head();
   	 for (int vert=0; vert<3; vert++)
	  {
		int ptId = GetPointId(trId, vert);

		for (int i=0; i<Neis[ptId].numNei; i++)
		{
          int curTr = Neis[ptId].pointNei[i];
          
		  if (curTr == trId ) continue;
		  
		  if( nei->Includes(curTr) ) {
			  nei->Tail();
              nei->Forward();
			  continue;
		  }         
		  nei->InsertAfter(curTr);
		  nei->Forward();
		}
	 }

}

void  polyData::Clear()
// Delete all data
{
  if (this->Points != NULL)
  {
   for ( int i=0; i<numPts; i++ )
	{
	  delete [] this->Points[i];
	}
    this->Points = NULL;
  }

  if (this->Triangles != NULL)
  {
   for ( int i=0; i<numTrs; i++ )
	{
	  delete [] this->Triangles[i];
	}

   this->Triangles = NULL;
  }
   
  if (this->Neis != NULL)
  {
   
  for (int i=0; i<numPts; i++)
   {
     delete [] this->Neis[i].pointNei;  
   }

    this->Neis = NULL;
  }
 
  if (this->nbhdArea)
  {
   delete [] nbhdArea; 
   this->nbhdArea = NULL;
  }

  if (this->trsArea)
  {
   delete [] trsArea;
   this->trsArea = NULL;
  }

}
