// renderer.cpp: implementation of the renderer class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "soft.h"
#include "renderer.h"
#include "Geometry2.h"
#include "Normals.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

//contour as points belonging to FACE and BACK tr-s 
//in the same time 
// 
void getContour3D(polyData *model, float *view, LList *contour)
{
  if (contour->length() != 0)
	 contour->Clear();
 
  int ptId1, ptId2;
  int *nei =  new int[5];
  float *n1 = new float[3];
  float *n2 = new float[3];
  
  float view3[3] = { view[0], view[1], view[2]};

  contour->Head();
  for ( int trId=1; trId<model->numTrs; trId++ )
  {
	for (int k=0; k<3; k++)
	{
	 ptId1 = model->GetPointId(trId,k);
	 ptId2 = model->GetPointId(trId,(k+1)%3);
     
     if (ptId1 > ptId2) continue;
     int n = model->GetEdgeNei(ptId1, ptId2, nei);
     
	 if (n != 2) continue;
     
	 int id0 = model->GetPointId(nei[0],0);
	 int id1 = model->GetPointId(nei[0],1);
	 int id2 = model->GetPointId(nei[0],2);	 
	 Geometry::trNormal(model->Points[id0], model->Points[id1],model->Points[id2], n1);

	 id0 = model->GetPointId(nei[1],0);
	 id1 = model->GetPointId(nei[1],1);
	 id2 = model->GetPointId(nei[1],2);	 
	 Geometry::trNormal(model->Points[id0], model->Points[id1],model->Points[id2], n2);

	 if ( (Geometry::dot(n1, view3) *Geometry::dot(n2, view3) <= 0) )  //||
		              //isZero( 0.5*dot(n1, view3)) || isZero( 0.5*dot(n2, view3)) )
	 {
       contour->InsertAfter(ptId1);
	   contour->Forward();
	   contour->InsertAfter(ptId2);
	   contour->Forward();
	 }
	}
  }
   delete [] nei;
   delete [] n1;
   delete [] n2;
 }

float getMinEdge(polyData *model)
{
  int ptId1, ptId2;
  float length =0.0;
  float *v = new float[3];
	
  ptId1 = model->GetPointId(0,0);
  ptId2 = model->GetPointId(0,1);

  v[0] = model->Points[ptId1][0] - model->Points[ptId2][0];
  v[1] = model->Points[ptId1][1] - model->Points[ptId2][1];
  v[2] = model->Points[ptId1][2] - model->Points[ptId2][2];

  length = Geometry::Norm(v);

  for ( int trId=1; trId<model->numTrs; trId++ )
  {
	ptId1 = model->GetPointId(trId,0);
	ptId2 = model->GetPointId(trId,1);
    if (ptId1 <ptId2)
	{
	  v[0] = model->Points[ptId1][0] - model->Points[ptId2][0];
      v[1] = model->Points[ptId1][1] - model->Points[ptId2][1];
      v[2] = model->Points[ptId1][2] - model->Points[ptId2][2];

	  if (Geometry::Norm(v) < length) length = Geometry::Norm(v);
	}

	ptId1 = model->GetPointId(trId,1);
	ptId2 = model->GetPointId(trId,2);
    if (ptId1 <ptId2)
	{
	  v[0] = model->Points[ptId1][0] - model->Points[ptId2][0];
      v[1] = model->Points[ptId1][1] - model->Points[ptId2][1];
      v[2] = model->Points[ptId1][2] - model->Points[ptId2][2];

	  if (Geometry::Norm(v) < length) length = Geometry::Norm(v);
	}

	ptId1 = model->GetPointId(trId,2);
	ptId2 = model->GetPointId(trId,0);
    if (ptId1 <ptId2)
	{
	  v[0] = model->Points[ptId1][0] - model->Points[ptId2][0];
      v[1] = model->Points[ptId1][1] - model->Points[ptId2][1];
      v[2] = model->Points[ptId1][2] - model->Points[ptId2][2];

	  if (Geometry::Norm(v) < length) length = Geometry::Norm(v);
	}
  }

	return length;
}



// as Zorin SIGGRAPH 2000
//my normals oriented inside, use -n compearing to Zorin's
//
void silhouette(polyData *model, float *viewDir, LList *silhoette)
{
	float N[4];
    int i;
    float c_max; 

	Normals nmls = Normals(model->numPts);
	nmls.setInput(model);
	nmls.Proceed();

	float **Dual;
	Dual = new float* [model->numPts];
	for (i=0; i< model->numPts; i++)
	{
      Dual[i] = new float[4]; 
	}

	for (int ptId=0; ptId<model->numPts; ptId++)
	{
      N[0] = -nmls.getOutput()[ptId][0];
	  N[1] = -nmls.getOutput()[ptId][1];
      N[2] = -nmls.getOutput()[ptId][2];    
	  N[3] =  Geometry::dot(viewDir, nmls.getOutput()[ptId]);

	  c_max = fabs(N[0]);
	  if ( fabs(N[1]) > c_max) c_max =  fabs(N[1]);
	  if ( fabs(N[2]) > c_max) c_max =  fabs(N[2]);
	  if ( fabs(N[4]) > c_max) c_max =  fabs(N[4]);

      for (int k=0; k<4; k++)
	  {
       Dual[ptId][k] = N[k] / c_max; 
	  }

	}


	for (i=0; i< model->numPts; i++)
	{
      delete [] Dual[i];
	}
	delete [] Dual;

}


void sketchRavine(float **field, CSoftDoc * doc, float *viewDir, float strokeLength, 
					                  float strokeWidth, float m_featureThr)
{  
	
   GLfloat s = strokeLength;
   float scale;

   Geometry::normalize(viewDir);
   float T[3];

   glLineWidth(strokeWidth);
   glBegin( GL_LINES ); 
   glColor3f(0.0f, 0.0f, 0.0f);
   doc->feature->ravine.Head(); 
   for (int count=0; count<doc->feature->ravine.length(); count++ )
	{
       int ptId = doc->feature->ravine.Retrieve();
	   if (doc->modelCurv->Kmin[ptId] > -m_featureThr)
	   {
	     doc->feature->ravine.Forward(); 
		 continue;
	   }

	   if ( !isOnFrontFacing(ptId, doc->modelPtr, viewDir) ) {
		   doc->feature->ravine.Forward(); 
		   continue;
	   }

       if ( field[ptId][0] == 0 && field[ptId][1] == 0 &&
		                          field[ptId][2] == 0)  {
	            doc->feature->ravine.Forward(); 
				continue;                     //umbilic
	   }

       scale = Geometry::dot(viewDir, field[ptId]);
	   if (Geometry::isZero(1.0 - scale)) {
		   doc->feature->ravine.Forward(); 
		   continue;                          //field is parallel to the view
	   }

	   for (int k=0; k<3; k++)
	   {
        T[k] = field[ptId][k] - scale * viewDir[k];
	   }
      

       float ss = s/Geometry::Norm(T);

       glVertex3f((GLfloat) doc->modelPtr->Points[ptId][0],
			         (GLfloat) doc->modelPtr->Points[ptId][1],
					 (GLfloat) doc->modelPtr->Points[ptId][2]);
	   glVertex3f((GLfloat) doc->modelPtr->Points[ptId][0] + ss*field[ptId][0],
			         (GLfloat) doc->modelPtr->Points[ptId][1] + ss*field[ptId][1],
				 (GLfloat) doc->modelPtr->Points[ptId][2] + ss*field[ptId][2]);
       doc->feature->ravine.Forward();  
   }
    glEnd();
}


void sketchRidge(float **field, polyData *modelPtr, float *viewDir,
				       LList ridge, curvature *curv, float strokeLength, 		
					                         float strokeWidth, float m_featureThr)
{  
	
   GLfloat s = strokeLength;
   float scale;

   Geometry::normalize(viewDir);
   float T[3];

   glLineWidth(strokeWidth);
   glBegin( GL_LINES ); 
   glColor3f(0.0f, 0.0f, 0.0f);
   ridge.Head(); 
   for (int count=0; count<ridge.length(); count++ )
	{
       int ptId = ridge.Retrieve();
	   if ( curv->Kmax[ptId] < m_featureThr)
	   {
	     ridge.Forward(); 
		 continue;
	   }

	   if ( !isOnFrontFacing(ptId, modelPtr, viewDir) ) {
		   ridge.Forward(); 
		   continue;
	   }

       if ( field[ptId][0] == 0 && field[ptId][1] == 0 &&
		                          field[ptId][2] == 0)  {
	            ridge.Forward(); 
				continue;                     //umbilic
	   }

       scale = Geometry::dot(viewDir, field[ptId]);
	   if (Geometry::isZero(1.0 - scale)) {
		   ridge.Forward(); 
		   continue;                          //field is parallel to the view
	   }

	   for (int k=0; k<3; k++)
	   {
        T[k] = field[ptId][k] - scale * viewDir[k];
	   }
      

       float ss = s/Geometry::Norm(T);
	
       glVertex3f((GLfloat) modelPtr->Points[ptId][0],
			         (GLfloat) modelPtr->Points[ptId][1],
					 (GLfloat) modelPtr->Points[ptId][2]);
	   glVertex3f((GLfloat) modelPtr->Points[ptId][0] + ss*field[ptId][0],
			         (GLfloat) modelPtr->Points[ptId][1] + ss*field[ptId][1],
				 (GLfloat) modelPtr->Points[ptId][2] + ss*field[ptId][2]);
       ridge.Forward();  
   }
    glEnd();
}


BOOL isOnFrontFacing(int ptId, polyData *model, float *view)
{
 
  BOOL test = FALSE;

  int num = model->Neis[ptId].numNei; 

  for (int i =0; i<num; i++)
  {
    int trId = model->Neis[ptId].pointNei[i];

	if (isFrontFacing(trId, model, view)) 
	{
	  test = TRUE;	break; 
	}

  }

  return test;
}

BOOL isFrontFacing(int trId, polyData *model, float *view)
{
 
  int p0, p1, p2;
  float n[3];

  BOOL test = FALSE;

  p0 = model->GetPointId(trId,0);
  p1 = model->GetPointId(trId,1);
  p2 = model->GetPointId(trId,2);

  Geometry::trNormal(model->Points[p0], model->Points[p1], model->Points[p2], n);

  if ( Geometry::dot(n, view) >= 0) 
	{
	  test = TRUE;
	}

  return test;
}


void sketchContour(LList *contour, float *view, polyData *modelPtr, 
				          float strokeLength, float strokeWidth)
{
	float ss = strokeLength;
	float tang[3] = {0, 0, 0};
    float n[3];

    contour->Head();
    glLineWidth(strokeWidth);
	glBegin(GL_LINES);
    contour->Head();	
	for (int count=0; count<contour->length(); count++)
	{
	  int ptId = contour->Retrieve();

	  Normals::getVertexNormal(ptId, modelPtr, n);
 	  Geometry::cross(view, n, tang);
	  Geometry::normalize(tang);

      glVertex3f((GLfloat) modelPtr->Points[ptId][0],
			         (GLfloat) modelPtr->Points[ptId][1],
					 (GLfloat) modelPtr->Points[ptId][2]);
	   glVertex3f((GLfloat) modelPtr->Points[ptId][0] + ss*tang[0],
			         (GLfloat) modelPtr->Points[ptId][1] + ss*tang[1],
				 (GLfloat) modelPtr->Points[ptId][2] + ss*tang[2]);

       contour->Forward();  

	}

    glEnd();
}


