
const YES = 1;
const NO = 0;

float T =0.0;

extern void cross( float *v1, float *v2, float *cross );
extern int ifIntersect(float *Qi, float *Qii, float *planeNormal, 
					                             float *planePoint);
extern void line_and_plane(float *s, float *lambda, int parallel, 
					float *a, float *ab, float *planeNormal, float *planePoint);
float dKmax_dTmin(curvature *Curv, int ptId, polyData *modelPtr);


void getRidge(polyModel *model, curvature *C,   LList *ridge)
{
  float *p1, *p2, *p0;
  float *lambda = new float[1];
  int parallel;
  float lineDir[3];
  int ridgePoint;
  int numInc;

  int *inc = new int[100];
  float *A = new float[3];

  for (int i=0; i<model->numPts; i++)                   //for all points
  { 
	if (C->Kmax[i] <0) continue;
    p0 = model->Points[i];
    ridgePoint = YES;

    numInc = model->incidentVts(i, inc);
    int isExotic = 0;
    for (int j=0; j<numInc; j++)
	{
	  p1 = model->Points[inc[j]];
      p2 = model->Points[inc[(j+1)%numInc]];  
	  
      if ( ifIntersect(p1, p2, C->Tmin[i], p0) == 1 )
	   {
		 isExotic ++;

		 lineDir[0] = p1[0]-p2[0];
		 lineDir[1] = p1[1]-p2[1];
		 lineDir[2] = p1[2]-p2[2];

         line_and_plane(A, lambda, parallel, p1, lineDir, C->Tmin[i], p0);

		 float KA = C->intrpolate(A, inc[j], inc[(j+1)%numInc], model, C->Kmax);

		 float Kt = dKmax_dTmin(C, i, model);
		   
		 float *v = new float[3];
		 v[0] = A[0] - p0[0];
		 v[1] = A[1] - p0[1];
		 v[2] = A[2] - p0[2];

		 float PA = norm(v);

		 float newKA = KA + 0.5* Kt*Kt *PA*PA /(C->Kmax[i] - C->Kmin[i]);

         if ( (newKA > C->Kmax[i]) ) { ridgePoint = NO; j = numInc;}
	   }      //end ifIntersect
	}  // for all neighbors
  
	
	if ( (ridgePoint == YES) && (C->Kmax[i] >T )) ridge->InsertAfter(i); 
  }    // for all point     
  
  delete inc;
  delete A;
}


void ridgeConnect(LList *ridgePtr, polyModel *modelPtr, LList *ridgeEdges)
{
  int *inc = new int[20];
  int p, p1, p2, numInc, num, count;
  num = ridgePtr->length();
  count=0;
  ridgeEdges->Head();
  while( count<num-1)
  {
    ridgePtr->Head();
	for (int temp =0; temp <count; temp++)
	{
     ridgePtr->Forward();
	}

	p1 = ridgePtr->Retrieve();
    numInc = modelPtr->incidentVts(p1, inc);
	for (int curInc=0; curInc<numInc; curInc++)
	{
      p2 = inc[curInc];
      ridgePtr->Head();
      for (int j= 0; j<	num; j++)
	  {
       p = ridgePtr->Retrieve(); 
	   if ( p2 == p )
	   { 
	     ridgeEdges->InsertAfter(p1);
         ridgeEdges->Forward();
         ridgeEdges->InsertAfter(p2);
         ridgeEdges->Forward();
	   }
       ridgePtr->Forward();
	  }
	 
	}
    count++;
  }
  delete inc;
}  


//
//return a directional direvative up tu sign
//

float dKmax_dTmin(curvature *Curv, int ptId, polyModel *modelPtr)
{ 
  float *p1, *p2, *p0;
  float *lambda = new float[1];
  int parallel;
  float lineDir[3];
  int numInc;
  float der;

  int *inc = new int[100];
  float **CD = new float*[2];
  for (int i=0; i<2; i++)
  {
	CD[i]= new float[2];
  }
  
  float *K_CD = new float[2];

  p0 = modelPtr->Points[ptId];

  numInc = modelPtr->incidentVts(ptId, inc);
  i=0;
  for (int j=0; j<numInc; j++)
  {
	p1 = modelPtr->Points[inc[j]];
    p2 = modelPtr->Points[inc[(j+1)%numInc]];

    if ( ifIntersect(p1, p2, Curv->Tmax[ptId], p0) == 1 )
	{
		lineDir[0] = p1[0]-p2[0];
		lineDir[1] = p1[1]-p2[1];
		lineDir[2] = p1[2]-p2[2];

        line_and_plane(CD[i], lambda, parallel, p1, lineDir, Curv->Tmax[ptId], p0);

        K_CD[i] = Curv->intrpolate(CD[i], inc[j], inc[(j+1)%numInc], modelPtr, Curv->Kmax);
	  	i++;
		if (i==2) j = numInc;
	}

	float *temp = new float[3];
	temp[0] = p0[0] - CD[0][0];
	temp[1] = p0[1] - CD[0][1];
	temp[2] = p0[2] - CD[0][2];

	float p0C = norm(temp);

	temp[0] = p0[0] - CD[1][0];
	temp[1] = p0[1] - CD[1][1];
	temp[2] = p0[2] - CD[1][2];

	float p0D = norm(temp);

    der = (K_CD[0] - Curv->Kmax[ptId])/p0C +
                      (Curv->Kmax[ptId] - K_CD[1])/p0D -
					  (K_CD[0] - K_CD[1])/(p0C+p0D);
  }
	return der;
}