// softView.h : interface of the CSoftView class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_SOFTVIEW_H__D975A1B9_F6B7_11D4_B299_0000F45E136F__INCLUDED_)
#define AFX_SOFTVIEW_H__D975A1B9_F6B7_11D4_B299_0000F45E136F__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


#include <gl\gl.h>
#include <gl\glu.h>
#include "BrushOptionsDlg.h"

enum {smooth, flat, wire, on, off, sketch, lstRavine,
            lstSmooth, lstFlat, lstRidge, lstWireOnly, 
			                lstHandRidge, lstHandRavine, lstContour};

class CSoftView : public CView
{
protected: // create from serialization only
	CSoftView();
	DECLARE_DYNCREATE(CSoftView)

// Attributes 
public:
	CSoftDoc* GetDocument();


protected:
    // Rotation amounts
    GLfloat xRot;
    GLfloat yRot;
	GLfloat distance;
	float distanceFactor;
	float spinFactor;
	bool distanceMode;
	CPoint mouseStartPos;
	bool movementMode;
	CBrushOptionsDlg* m_brushDlg;

private:
	int m_pixelFormat;
	HGLRC m_hRC;
	CDC * m_pDC;
	int m_drawMode;
	int m_ridge;
	int m_ravine;
	int m_both;
	int m_surface;
    int m_contour; 
	float m_featureThr;
	float m_strokeWidth;
	float m_strokeLength;
	float zoom;
    BOOL BrushDlgShowing;

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CSoftView)
	public:
	virtual void OnDraw(CDC* pDC);  // overridden to draw this view
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	//}}AFX_VIRTUAL

// Implementation
public:
	void InitView();
	virtual ~CSoftView();

   void drawModel(polyData *modelPtr);
   void SetupViewVolume(polyData *model);

#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:
   BOOL SetUpGL(void);
   BOOL SetupPixelFormat();
   void drawSmooth(polyData *model);
   void drawWireframe(polyData *model);
   void flatShading(polyData *model);
   void drawRidge(polyData *modelPtr, LList &ridge, float m_featureThr);  
   void drawRavine(polyData *modelPtr, LList &ravine, float m_featureThr);
   void drawWhiteModel(polyData *model);
   void drawContour3D(polyData *modelPtr);
   void getViewDir(float *newView);
   void DrawSketch(CSoftDoc *doc); 
   
// Generated message map functions
protected:
	//{{AFX_MSG(CSoftView)
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnDestroy();
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg BOOL OnEraseBkgnd(CDC* pDC);
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnViewoptionFlatshading();
	afx_msg void OnUpdateViewStatusBar(CCmdUI* pCmdUI);
	afx_msg void OnViewoptionSmoothshading();
	afx_msg void OnUpdateViewoptionSmoothshading(CCmdUI* pCmdUI);
	afx_msg void OnUpdateViewoptionFlatshading(CCmdUI* pCmdUI);
	afx_msg void OnViewoptionWireframe();
	afx_msg void OnUpdateViewoptionWireframe(CCmdUI* pCmdUI);
	afx_msg void OnDblclkModellist();
	afx_msg void OnFeaturesRidges();
	afx_msg void OnUpdateFeaturesRidges(CCmdUI* pCmdUI);
	afx_msg void OnFeaturesRavines();
	afx_msg void OnUpdateFeaturesRavines(CCmdUI* pCmdUI);
	afx_msg void OnFeaturesSnowsurface();
	afx_msg void OnUpdateFeaturesSnowsurface(CCmdUI* pCmdUI);
    afx_msg void OnDrawSketch();
	afx_msg void OnContour();
	afx_msg void OnSubdivide();
	afx_msg void OnApply();
	afx_msg void OnReset();
	afx_msg void OnChooseBrush();
	//}}AFX_MSG
    afx_msg LRESULT OnViewApplyBrush(WPARAM, LPARAM);
    afx_msg LRESULT OnViewCloseBrushDlg(WPARAM, LPARAM);
	DECLARE_MESSAGE_MAP()
};

#ifndef _DEBUG  // debug version in softView.cpp
inline CSoftDoc* CSoftView::GetDocument()
   { return (CSoftDoc*)m_pDocument; }
#endif

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_SOFTVIEW_H__D975A1B9_F6B7_11D4_B299_0000F45E136F__INCLUDED_)
